@@ -5,14 +5,20 @@
 
 #pragma once
 
+#include "keyverificationsession.h"
 #include "quotient_common.h"
 #include "ssosession.h"
 #include "util.h"
 
 #include "csapi/create_room.h"
 #include "csapi/login.h"
+#include "csapi/content-repo.h"
+
+#include "e2ee/qolmoutboundsession.h"
 
 #include "events/accountdataevents.h"
+#include "jobs/jobhandle.h"
+#include "jobs/syncjob.h"
 
 #include <QtCore/QDir>
 #include <QtCore/QObject>
@@ -21,15 +27,10 @@
 
 #include <functional>
 
-#ifdef Quotient_E2EE_ENABLED
-#include "e2ee/e2ee_common.h"
-#include "e2ee/qolmoutboundsession.h"
-#include "keyverificationsession.h"
-#include "events/keyverificationevent.h"
-#endif
-
 Q_DECLARE_METATYPE(Quotient::GetLoginFlowsJob::LoginFlow)
 
+class TestCrossSigning;
+
 namespace Quotient {
 
 class Avatar;
@@ -38,15 +39,13 @@ class User;
 class ConnectionData;
 class RoomEvent;
 
-class SyncJob;
-class SyncData;
+class GetVersionsJob;
+class GetCapabilitiesJob;
 class RoomMessagesJob;
 class PostReceiptJob;
 class ForgetRoomJob;
 class MediaThumbnailJob;
 class JoinRoomJob;
-class UploadContentJob;
-class GetContentJob;
 class DownloadFileJob;
 class SendToDeviceJob;
 class SendMessageJob;
@@ -61,9 +60,9 @@ using LoginFlow = GetLoginFlowsJob::LoginFlow;
 
 //! Predefined login flows
 namespace LoginFlows {
-    inline const LoginFlow Password { "m.login.password"_ls };
-    inline const LoginFlow SSO { "m.login.sso"_ls };
-    inline const LoginFlow Token { "m.login.token"_ls };
+    inline const LoginFlow Password { "m.login.password"_L1 };
+    inline const LoginFlow SSO { "m.login.sso"_L1 };
+    inline const LoginFlow Token { "m.login.token"_L1 };
 }
 
 // To simplify comparisons of LoginFlows
@@ -109,7 +108,6 @@ auto defaultUserFactory(Connection* c, const QString& id)
 // rooms in Join and Leave state; and direct chats in account data
 // are stored with no regard to their state.
 using DirectChatsMap = QMultiHash<const User*, QString>;
-using DirectChatUsersMap = QMultiHash<QString, User*>;
 using IgnoredUsersList = IgnoredUsersEvent::value_type;
 
 class QUOTIENT_API Connection : public QObject {
@@ -121,6 +119,7 @@ class QUOTIENT_API Connection : public QObject {
     Q_PROPERTY(QString deviceId READ deviceId NOTIFY stateChanged)
     Q_PROPERTY(QByteArray accessToken READ accessToken NOTIFY stateChanged)
     Q_PROPERTY(bool isLoggedIn READ isLoggedIn NOTIFY stateChanged STORED false)
+    Q_PROPERTY(bool isOnline READ isOnline NOTIFY isOnlineChanged)
     Q_PROPERTY(QString defaultRoomVersion READ defaultRoomVersion NOTIFY capabilitiesLoaded)
     Q_PROPERTY(QUrl homeserver READ homeserver WRITE setHomeserver NOTIFY homeserverChanged)
     Q_PROPERTY(QVector<GetLoginFlowsJob::LoginFlow> loginFlows READ loginFlows NOTIFY loginFlowsChanged)
@@ -238,14 +237,6 @@ public:
     //! \sa directChatsListChanged
     void addToDirectChats(const Room* room, const QString& userId);
 
-    //! \brief Mark the room as a direct chat with the user
-    //!
-    //! This function marks \p room as a direct chat with \p user.
-    //! Emits the signal synchronously, without waiting to complete
-    //! synchronisation with the server.
-    //! \sa directChatsListChanged
-    void addToDirectChats(const Room* room, User* user);
-
     //! \brief Unmark the room from direct chats
     //!
     //! This function removes the room id from direct chats either for
@@ -257,17 +248,6 @@ public:
     //! \sa directChatsListChanged
     void removeFromDirectChats(const QString& roomId, const QString& userId = {});
 
-    //! \brief Unmark the room from direct chats
-    //!
-    //! This function removes the room id from direct chats either for
-    //! a specific \p user or for all users if \p user in nullptr.
-    //! The room id is used to allow removal of, e.g., ids of forgotten
-    //! rooms; a Room object need not exist. Emits the signal
-    //! immediately, without waiting to complete synchronisation with
-    //! the server.
-    //! \sa directChatsListChanged
-    void removeFromDirectChats(const QString& roomId, User* user = nullptr);
-
     //! Check whether the room id corresponds to a direct chat
     bool isDirectChat(const QString& roomId) const;
 
@@ -280,28 +260,16 @@ public:
     //!         a direct chat; an empty list if the room is not a direct chat
     QList<QString> directChatMemberIds(const Room* room) const;
 
-    //! \brief Retrieve the list of users the room is a direct chat with
-    //! \return The list of users for which this room is marked as
-    //!         a direct chat; an empty list if the room is not a direct chat
-    QList<User*> directChatUsers(const Room* room) const;
-
     //! Check whether a particular user id is in the ignore list
     Q_INVOKABLE bool isIgnored(const QString& userId) const;
 
     //! Check whether a particular user is in the ignore list
+    [[deprecated("Use the overload accepting UserId instead")]]
     Q_INVOKABLE bool isIgnored(const Quotient::User* user) const;
 
     //! Get the whole list of ignored users
     Q_INVOKABLE Quotient::IgnoredUsersList ignoredUsers() const;
 
-    //! \brief Add the user to the ignore list
-    //! The change signal is emitted synchronously, without waiting
-    //! to complete synchronisation with the server.
-    //!
-    //! \sa ignoredUsersListChanged
-    [[deprecated("Use the overload that accepts a user id instead")]] //
-    Q_INVOKABLE void addToIgnoredUsers(const Quotient::User* user);
-
     //! \brief Add the user to the ignore list
     //! The change signal is emitted synchronously, without waiting
     //! to complete synchronisation with the server.
@@ -309,27 +277,23 @@ public:
     //! \sa ignoredUsersListChanged
     Q_INVOKABLE void addToIgnoredUsers(const QString& userId);
 
-    //! \brief Remove the user from the ignore list
-    //!
-    //! Similar to adding, the change signal is emitted synchronously.
-    //! \sa ignoredUsersListChanged
-    [[deprecated("Use the overload that accepts a user id instead")]] //
-    Q_INVOKABLE void removeFromIgnoredUsers(const Quotient::User* user);
-
     //! \brief Remove the user from the ignore list
     //!
     //! Similar to adding, the change signal is emitted synchronously.
     //! \sa ignoredUsersListChanged
     Q_INVOKABLE void removeFromIgnoredUsers(const QString& userId);
 
-    //! Get the full list of users known to this account
-    QMap<QString, User*> users() const;
+    //! \brief Get the entire list of users known to the current user on this homeserver
+    //! \note Be mindful that this can easily count thousands or tens of thousands, and use
+    //!       sparingly; when in a room context, always use Room::members() instead
+    Q_INVOKABLE QStringList userIds() const;
 
     //! Get the base URL of the homeserver to connect to
     QUrl homeserver() const;
     //! Get the domain name used for ids/aliases on the server
     QString domain() const;
     //! Check if the homeserver is known to be reachable and working
+    [[deprecated("Check the result returned by Connection::loginFlows() instead")]]
     bool isUsable() const;
     //! Get the list of supported login flows
     QVector<GetLoginFlowsJob::LoginFlow> loginFlows() const;
@@ -368,23 +332,23 @@ public:
     QString deviceId() const;
     QByteArray accessToken() const;
     bool isLoggedIn() const;
-#ifdef Quotient_E2EE_ENABLED
+
+    //! \brief Whether the connection is successfully syncing with the server.
+    //!
+    //! \return true, if the last sync was successful, false otherwise.
+    bool isOnline() const;
     QOlmAccount* olmAccount() const;
     Database* database() const;
 
-    UnorderedMap<QByteArray, QOlmInboundGroupSession> loadRoomMegolmSessions(
+    std::unordered_map<QByteArray, QOlmInboundGroupSession> loadRoomMegolmSessions(
         const Room* room) const;
     void saveMegolmSession(const Room* room,
-                           const QOlmInboundGroupSession& session) const;
-    [[deprecated("Use database()->loadCurrentOutboundMegolmSession()")]]
-    Omittable<QOlmOutboundGroupSession> loadCurrentOutboundMegolmSession(
-        const QString& roomId) const;
-    [[deprecated("Use database()->saveCurrentOutboundMegolmSession()")]]
-    void saveCurrentOutboundMegolmSession(
-        const QString& roomId, const QOlmOutboundGroupSession& session) const;
+                           const QOlmInboundGroupSession& session, const QByteArray &senderKey, const QByteArray& senderEdKey) const;
 
     QString edKeyForUserDevice(const QString& userId,
                                const QString& deviceId) const;
+    QString curveKeyForUserDevice(const QString& userId,
+                                const QString& device) const;
     bool hasOlmSession(const QString& user, const QString& deviceId) const;
 
     // This assumes that an olm session already exists. If it doesn't, no message is sent.
@@ -412,9 +376,14 @@ public:
     QStringList devicesForUser(const QString& userId) const;
     Q_INVOKABLE bool isQueryingKeys() const;
 
-    void requestKeyFromDevices(
-        event_type_t name, const std::function<void(const QByteArray&)>& then = [](auto) {});
-#endif // Quotient_E2EE_ENABLED
+    QFuture<QByteArray> requestKeyFromDevices(event_type_t name);
+
+    QString masterKeyForUser(const QString& userId) const;
+    Q_INVOKABLE bool isUserVerified(const QString& userId) const;
+    Q_INVOKABLE bool allSessionsSelfVerified(const QString& userId) const;
+    bool hasConflictingDeviceIdsAndCrossSigningKeys(const QString& userId);
+
+    void reloadDevices();
 
     Q_INVOKABLE Quotient::SyncJob* syncJob() const;
     Q_INVOKABLE QString nextBatchToken() const;
@@ -426,7 +395,7 @@ public:
         QString id;
         QString status;
 
-        static const QString StableTag; // "stable", as of CS API 0.5
+        static constexpr QStringView StableTag = u"stable";
         bool isStable() const { return status == StableTag; }
 
         friend QDebug operator<<(QDebug dbg, const SupportedRoomVersion& v)
@@ -436,9 +405,15 @@ public:
         }
     };
 
-    //! Find out if capabilites are still loading from the server
+    //! Find out if homeserver capabilites have been loaded
+    Q_INVOKABLE bool capabilitiesReady() const;
+
+    [[deprecated("Use capabilitiesReady() instead; don't forget to negate the returned value")]]
     Q_INVOKABLE bool loadingCapabilities() const;
 
+    //! Get the list of Matrix CS API spec versions supported by the homeserver
+    QStringList supportedMatrixSpecVersions() const;
+
     //! \brief Get the room version recommended by the server
     //!
     //! Only works after server capabilities have been loaded.
@@ -460,15 +435,6 @@ public:
     bool canChangePassword() const;
 
     //! \brief Check whether encryption is enabled on this connection
-    //!
-    //! There are two conditions for encryption to be enabled:
-    //! 1) the library must be compiled with Quotient_E2EE_ENABLED;
-    //! 2) encryption should be switched on explicitly with enableEncryption()
-    //!
-    //! The reason for the latter is E2EE still being in beta state as of 0.8,
-    //! so clients are strongly advised to keep E2EE off by default and warn
-    //! users opting in that their E2EE things may fall apart, leak etc.
-    //!
     //! \sa enableEncryption
     bool encryptionEnabled() const;
 
@@ -535,8 +501,21 @@ public:
     void setLazyLoading(bool newValue);
 
     //! Start a pre-created job object on this connection
-    Q_INVOKABLE BaseJob* run(BaseJob* job,
-                             RunningPolicy runningPolicy = ForegroundRequest);
+    Q_INVOKABLE BaseJob* run(BaseJob* job, RunningPolicy runningPolicy = ForegroundRequest);
+
+    //! \brief Start a pre-created job on this connection and get a job handle to it
+    //!
+    //! This is a template overload for run(BaseJob*, RunningPolicy) - if you call run() on any
+    //! derived job (99% of the cases when you're going to call it), this overload will be chosen
+    //! as a more type-safe and feature-rich version. It's not Q_INVOKABLE though.
+    template <std::derived_from<BaseJob> JobT>
+        requires (!std::same_as<JobT, BaseJob>)
+    JobHandle<JobT> run(JobT* job, RunningPolicy runningPolicy = ForegroundRequest)
+    {
+        JobHandle jh { job };
+        run(static_cast<BaseJob*>(job), runningPolicy);
+        return jh;
+    }
 
     //! \brief Start a job of a given type with specified arguments and policy
     //!
@@ -550,21 +529,18 @@ public:
     //!
     //! \sa BaseJob::isBackground. QNetworkRequest::BackgroundRequestAttribute
     template <typename JobT, typename... JobArgTs>
-    JobT* callApi(RunningPolicy runningPolicy, JobArgTs&&... jobArgs)
+    JobHandle<JobT> callApi(RunningPolicy runningPolicy, JobArgTs&&... jobArgs)
     {
-        auto job = new JobT(std::forward<JobArgTs>(jobArgs)...);
-        run(job, runningPolicy);
-        return job;
+        return run(new JobT(std::forward<JobArgTs>(jobArgs)...), runningPolicy);
     }
 
     //! \brief Start a job of a specified type with specified arguments
     //!
     //! This is an overload that runs the job with "foreground" policy.
     template <typename JobT, typename... JobArgTs>
-    JobT* callApi(JobArgTs&&... jobArgs)
+    JobHandle<JobT> callApi(JobArgTs&&... jobArgs)
     {
-        return callApi<JobT>(ForegroundRequest,
-                             std::forward<JobArgTs>(jobArgs)...);
+        return callApi<JobT>(ForegroundRequest, std::forward<JobArgTs>(jobArgs)...);
     }
 
     //! \brief Get a request URL for a job with specified type and arguments
@@ -574,8 +550,7 @@ public:
     template <typename JobT, typename... JobArgTs>
     QUrl getUrlForApi(JobArgTs&&... jobArgs) const
     {
-        return JobT::makeRequestUrl(homeserver(),
-                                    std::forward<JobArgTs>(jobArgs)...);
+        return JobT::makeRequestUrl(homeserverData(), std::forward<JobArgTs>(jobArgs)...);
     }
 
     //! \brief Start a local HTTP server and generate a single sign-on URL
@@ -603,13 +578,11 @@ public:
     Q_INVOKABLE bool roomSucceeds(const QString& maybePredecessorId,
                                   const QString& maybeSuccessorId) const;
 
-#ifdef Quotient_E2EE_ENABLED
     //! Set the E2EE default state for any Connection created further
     static void setEncryptionDefault(bool useByDefault);
 
     //! Set the direct chat E2EE default state for any Connection created further
     static void setDirectChatEncryptionDefault(bool useByDefault);
-#endif
 
     //! Set a room factory function
     static void setRoomFactory(room_factory_t f);
@@ -637,12 +610,6 @@ public:
         setUserFactory(defaultUserFactory<T>);
     }
 
-public Q_SLOTS:
-    //! \brief Set the homeserver base URL and retrieve its login flows
-    //!
-    //! \sa LoginFlowsJob, loginFlows, loginFlowsChanged, homeserverChanged
-    void setHomeserver(const QUrl& baseUrl);
-
     //! \brief Determine and set the homeserver from MXID
     //!
     //! This attempts to resolve the homeserver by requesting
@@ -655,8 +622,33 @@ public Q_SLOTS:
     //! attempt.
     //! \param mxid user Matrix ID, such as @someone:example.org
     //! \sa setHomeserver, homeserverChanged, loginFlowsChanged, resolveError
-    void resolveServer(const QString& mxid);
+    Q_INVOKABLE void resolveServer(const QString& mxid);
 
+    //! \brief Set the homeserver base URL and retrieve its login flows
+    //!
+    //! \sa LoginFlowsJob, loginFlows, loginFlowsChanged, homeserverChanged
+    Q_INVOKABLE QFuture<QList<LoginFlow> > setHomeserver(const QUrl& baseUrl);
+
+    //! \brief Get a future to a direct chat with the user
+    Q_INVOKABLE QFuture<Room*> getDirectChat(const QString& otherUserId);
+
+    //! Create a direct chat with a single user, optional name and topic
+    //!
+    //! A room will always be created, unlike in requestDirectChat.
+    //! It is advised to use requestDirectChat as a default way of getting
+    //! one-on-one with a person, and only use createDirectChat when
+    //! a new creation is explicitly desired.
+    Q_INVOKABLE JobHandle<CreateRoomJob> createDirectChat(const QString& userId,
+                                                          const QString& topic = {},
+                                                          const QString& name = {});
+
+    Q_INVOKABLE JobHandle<JoinRoomJob> joinRoom(const QString& roomAlias,
+                                                const QStringList& serverNames = {});
+
+    Q_INVOKABLE QFuture<Room*> joinAndGetRoom(const QString& roomAlias,
+                                              const QStringList& serverNames = {});
+
+public Q_SLOTS:
     //! \brief Log in using a username and password pair
     //!
     //! Before logging in, this method checks if the homeserver is valid and
@@ -679,29 +671,27 @@ public Q_SLOTS:
                         const QString& initialDeviceName,
                         const QString& deviceId = {});
 
-    //! \brief Use an existing access token to connect to the homeserver
-    //!
-    //! This is the old signature that will be deprecated in 0.8; since 0.7.2,
-    //! device id is resolved from the access token. Use 2-arg assumeIdentity()
-    [[deprecated("Device id is now resolved from accessToken; use "
-                 "assumeIdentity(QString, QString) instead of this overload")]] //
-    void assumeIdentity(const QString& mxId, const QString& accessToken,
-                        [[maybe_unused]] const QString& deviceId);
-
     //! \brief Use an existing access token to connect to the homeserver
     //!
     //! Similar to loginWithPassword(), this method checks that the homeserver
     //! URL is valid and tries to resolve it from the MXID in case it is not.
     //! \since 0.7.2
-    void assumeIdentity(const QString& mxId, const QString& accessToken);
+    void assumeIdentity(const QString& mxId, const QString& deviceId, const QString& accessToken);
 
-    //! Explicitly request capabilities from the server
-    void reloadCapabilities();
+    //! \brief Request supported spec versions from the homeserver
+    //!
+    //! This call does not obtain room versions - use loadCapabilities() for that.
+    JobHandle<GetVersionsJob> loadVersions();
 
-    void logout();
+    //! Request capabilities and room versions from the server
+    JobHandle<GetCapabilitiesJob> loadCapabilities();
+
+    [[deprecated("Use loadCapabilities() instead")]] void reloadCapabilities();
+
+    QFuture<void> logout();
 
     void sync(int timeout = -1);
-    void syncLoop(int timeout = 30000);
+    void syncLoop(int timeout = SyncJob::defaultTimeoutMillis);
 
     void stopSync();
 
@@ -715,29 +705,25 @@ public Q_SLOTS:
                                     RunningPolicy policy = BackgroundRequest);
 
     // QIODevice* should already be open
-    UploadContentJob* uploadContent(QIODevice* contentSource,
-                                    const QString& filename = {},
-                                    const QString& overrideContentType = {});
-    UploadContentJob* uploadFile(const QString& fileName,
-                                 const QString& overrideContentType = {});
-    GetContentJob* getContent(const QString& mediaId);
-    GetContentJob* getContent(const QUrl& url);
+    JobHandle<UploadContentJob> uploadContent(QIODevice* contentSource, const QString& filename = {},
+                                              const QString& overrideContentType = {});
+    JobHandle<UploadContentJob> uploadFile(const QString& fileName,
+                                           const QString& overrideContentType = {});
+    [[deprecated("Use downloadFile() instead")]] BaseJob* getContent(const QString& mediaId);
+    [[deprecated("Use downloadFile() instead")]] BaseJob* getContent(const QUrl& url);
+
     // If localFilename is empty, a temporary file will be created
-    DownloadFileJob* downloadFile(const QUrl& url,
-                                  const QString& localFilename = {});
+    DownloadFileJob* downloadFile(const QUrl& url, const QString& localFilename = {});
 
-#ifdef Quotient_E2EE_ENABLED
     DownloadFileJob* downloadFile(const QUrl& url,
                                   const EncryptedFileMetadata& fileMetadata,
                                   const QString& localFilename = {});
-#endif
 
     //! \brief Create a room (generic method)
     //!
     //! This method allows to customize room entirely to your liking,
     //! providing all the attributes the original CS API provides.
-    CreateRoomJob*
-    createRoom(RoomVisibility visibility, const QString& alias,
+    JobHandle<CreateRoomJob> createRoom(RoomVisibility visibility, const QString& alias,
                const QString& name, const QString& topic, QStringList invites,
                const QString& presetName = {}, const QString& roomVersion = {},
                bool isDirect = false,
@@ -753,44 +739,6 @@ public Q_SLOTS:
     //! \sa directChatAvailable
     void requestDirectChat(const QString& userId);
 
-    //! \brief Get a direct chat with a single user
-    //!
-    //! This method may return synchronously or asynchoronously depending
-    //! on whether a direct chat room with the respective person exists
-    //! already.
-    //! \sa directChatAvailable
-    void requestDirectChat(User* u);
-
-    //! \brief Run an operation in a direct chat with the user
-    //!
-    //! This method may return synchronously or asynchoronously depending
-    //! on whether a direct chat room with the respective person exists
-    //! already. Instead of emitting a signal it executes the passed
-    //! function object with the direct chat room as its parameter.
-    void doInDirectChat(const QString& userId,
-                        const std::function<void(Room*)>& operation);
-
-    //! \brief Run an operation in a direct chat with the user
-    //!
-    //! This method may return synchronously or asynchoronously depending
-    //! on whether a direct chat room with the respective person exists
-    //! already. Instead of emitting a signal it executes the passed
-    //! function object with the direct chat room as its parameter.
-    void doInDirectChat(User* u, const std::function<void(Room*)>& operation);
-
-    //! Create a direct chat with a single user, optional name and topic
-    //!
-    //! A room will always be created, unlike in requestDirectChat.
-    //! It is advised to use requestDirectChat as a default way of getting
-    //! one-on-one with a person, and only use createDirectChat when
-    //! a new creation is explicitly desired.
-    CreateRoomJob* createDirectChat(const QString& userId,
-                                    const QString& topic = {},
-                                    const QString& name = {});
-
-    virtual JoinRoomJob* joinRoom(const QString& roomAlias,
-                                  const QStringList& serverNames = {});
-
     //! \brief Send /forget to the server and delete room locally
     //!
     //! This method is in Connection, not in Room, since it's a
@@ -814,16 +762,14 @@ public Q_SLOTS:
     //! \deprecated Do not use this directly, use Room::leaveRoom() instead
     virtual LeaveRoomJob* leaveRoom(Room* room);
 
-#ifdef Quotient_E2EE_ENABLED
-    KeyVerificationSession* startKeyVerificationSession(const QString& userId,
-                                                        const QString& deviceId);
+    Quotient::KeyVerificationSession* startKeyVerificationSession(const QString& userId,
+                                                                  const QString& deviceId);
 
-    void encryptionUpdate(const Room* room, const QList<QString>& invitedIds);
-    void encryptionUpdate(const Room* room, const QList<User*>& invited = {});
-#endif
+    Q_INVOKABLE void startSelfVerification();
+    void encryptionUpdate(const Room* room, const QStringList& invitedIds = {});
 
     static Connection* makeMockConnection(const QString& mxId,
-                                          bool enableEncryption = E2EE_Enabled);
+                                          bool enableEncryption = true);
 
 Q_SIGNALS:
     //! \brief Initial server resolution has failed
@@ -846,6 +792,10 @@ Q_SIGNALS:
     //! accessToken - these properties normally only change at
     //! a successful login and logout and are constant at other times.
     void stateChanged();
+
+    //! The online state has changed.
+    void isOnlineChanged();
+
     void loginError(QString message, QString details);
 
     //! \brief A network request (job) started by callApi() has failed
@@ -979,20 +929,32 @@ Q_SIGNALS:
     void encryptionChanged(bool enabled);
     void directChatsEncryptionChanged(bool enabled);
 
-#ifdef Quotient_E2EE_ENABLED
     void newKeyVerificationSession(Quotient::KeyVerificationSession* session);
     void keyVerificationStateChanged(
         const Quotient::KeyVerificationSession* session,
         Quotient::KeyVerificationSession::State state);
     void sessionVerified(const QString& userId, const QString& deviceId);
-    bool finishedQueryingKeys();
+    void finishedQueryingKeys();
     void secretReceived(const QString& requestId, const QString& secret);
-#endif
 
+    void userVerified(const QString& userId);
+
+    //! The account does not yet have cross-signing keys. The client should ask the user
+    //! whether to create them now and then set them up, if desired.
+    void crossSigningSetupRequired();
+
+    //! The connection is ready to be used. Most notably, the fundamental e2ee data is loaded.
+    //! This does not mean that the server was reached, a sync was performed, or the state cache was loaded.
+    void ready();
+
+    friend class ::TestCrossSigning;
 protected:
     //! Access the underlying ConnectionData class
     const ConnectionData* connectionData() const;
 
+    //! Get the homeserver data necessary to construct network requests
+    HomeserverData homeserverData() const;
+
     //! \brief Get a Room object for the given id in the given state
     //!
     //! Use this method when you need a Room object in the local list
@@ -1004,15 +966,14 @@ protected:
     //! will be deleted and a new room object with Join state created.
     //! In contrast, switching between Join and Leave happens within
     //! the same object.
-    //! \param roomId room id (not alias!)
+    //! \param id room id (not alias!)
     //! \param joinState desired (target) join state of the room; if
     //! omitted, any state will be found and return unchanged, or a
     //! new Join room created.
     //! \return a pointer to a Room object with the specified id and the
     //! specified state; nullptr if roomId is empty or if roomFactory()
     //! failed to create a Room object.
-    Room* provideRoom(const QString& roomId,
-                      Omittable<JoinState> joinState = none);
+    Room* provideRoom(const QString& id, std::optional<JoinState> joinState = {});
 
     //! Process sync data from a successful sync request
     void onSyncSuccess(SyncData&& data, bool fromCache = false);
