@@ -6,11 +6,12 @@
 
 namespace Quotient {
 //! Identification information for a user
-struct UserIdentifier {
-    //! The type of identification.  See [Identifier types](/client-server-api/#identifier-types)
+struct QUOTIENT_API UserIdentifier {
+    //! The type of identification. See [Identifier types](/client-server-api/#identifier-types)
     //! for supported values and additional property descriptions.
     QString type;
 
+    //! Keys dependent on the identification type.
     QVariantHash additionalProperties{};
 };
 
@@ -19,11 +20,11 @@ struct JsonObjectConverter<UserIdentifier> {
     static void dumpTo(QJsonObject& jo, const UserIdentifier& pod)
     {
         fillJson(jo, pod.additionalProperties);
-        addParam<>(jo, QStringLiteral("type"), pod.type);
+        addParam(jo, "type"_L1, pod.type);
     }
     static void fillFrom(QJsonObject jo, UserIdentifier& pod)
     {
-        fillFromJson(jo.take("type"_ls), pod.type);
+        fillFromJson(jo.take("type"_L1), pod.type);
         fromJson(jo, pod.additionalProperties);
     }
 };
