@@ -26,9 +26,11 @@ public:
     // Result properties
 
     //! The backup version. This is an opaque string.
-    QString version() const { return loadFromJson<QString>("version"_ls); }
+    QString version() const { return loadFromJson<QString>("version"_L1); }
 };
 
+inline auto collectResponse(const PostRoomKeysVersionJob* job) { return job->version(); }
+
 //! \brief Get information about the latest backup version.
 //!
 //! Get information about the latest backup version.
@@ -40,29 +42,57 @@ public:
     //!
     //! This function can be used when a URL for GetRoomKeysVersionCurrentJob
     //! is necessary but the job itself isn't.
-    static QUrl makeRequestUrl(QUrl baseUrl);
+    static QUrl makeRequestUrl(const HomeserverData& hsData);
 
     // Result properties
 
     //! The algorithm used for storing backups.
-    QString algorithm() const { return loadFromJson<QString>("algorithm"_ls); }
+    QString algorithm() const { return loadFromJson<QString>("algorithm"_L1); }
 
     //! Algorithm-dependent data. See the documentation for the backup
     //! algorithms in [Server-side key backups](/client-server-api/#server-side-key-backups) for
     //! more information on the expected format of the data.
-    QJsonObject authData() const { return loadFromJson<QJsonObject>("auth_data"_ls); }
+    QJsonObject authData() const { return loadFromJson<QJsonObject>("auth_data"_L1); }
 
     //! The number of keys stored in the backup.
-    int count() const { return loadFromJson<int>("count"_ls); }
+    int count() const { return loadFromJson<int>("count"_L1); }
 
     //! An opaque string representing stored keys in the backup.
     //! Clients can compare it with the `etag` value they received
     //! in the request of their last key storage request.  If not
     //! equal, another client has modified the backup.
-    QString etag() const { return loadFromJson<QString>("etag"_ls); }
+    QString etag() const { return loadFromJson<QString>("etag"_L1); }
 
     //! The backup version.
-    QString version() const { return loadFromJson<QString>("version"_ls); }
+    QString version() const { return loadFromJson<QString>("version"_L1); }
+
+    struct Response {
+        //! The algorithm used for storing backups.
+        QString algorithm{};
+
+        //! Algorithm-dependent data. See the documentation for the backup
+        //! algorithms in [Server-side key backups](/client-server-api/#server-side-key-backups) for
+        //! more information on the expected format of the data.
+        QJsonObject authData{};
+
+        //! The number of keys stored in the backup.
+        int count{};
+
+        //! An opaque string representing stored keys in the backup.
+        //! Clients can compare it with the `etag` value they received
+        //! in the request of their last key storage request.  If not
+        //! equal, another client has modified the backup.
+        QString etag{};
+
+        //! The backup version.
+        QString version{};
+    };
+};
+
+template <std::derived_from<GetRoomKeysVersionCurrentJob> JobT>
+constexpr inline auto doCollectResponse<JobT> =
+    [](JobT* j) -> GetRoomKeysVersionCurrentJob::Response {
+    return { j->algorithm(), j->authData(), j->count(), j->etag(), j->version() };
 };
 
 //! \brief Get information about an existing backup.
@@ -82,29 +112,56 @@ public:
     //!
     //! This function can be used when a URL for GetRoomKeysVersionJob
     //! is necessary but the job itself isn't.
-    static QUrl makeRequestUrl(QUrl baseUrl, const QString& version);
+    static QUrl makeRequestUrl(const HomeserverData& hsData, const QString& version);
 
     // Result properties
 
     //! The algorithm used for storing backups.
-    QString algorithm() const { return loadFromJson<QString>("algorithm"_ls); }
+    QString algorithm() const { return loadFromJson<QString>("algorithm"_L1); }
 
     //! Algorithm-dependent data. See the documentation for the backup
     //! algorithms in [Server-side key backups](/client-server-api/#server-side-key-backups) for
     //! more information on the expected format of the data.
-    QJsonObject authData() const { return loadFromJson<QJsonObject>("auth_data"_ls); }
+    QJsonObject authData() const { return loadFromJson<QJsonObject>("auth_data"_L1); }
 
     //! The number of keys stored in the backup.
-    int count() const { return loadFromJson<int>("count"_ls); }
+    int count() const { return loadFromJson<int>("count"_L1); }
 
     //! An opaque string representing stored keys in the backup.
     //! Clients can compare it with the `etag` value they received
     //! in the request of their last key storage request.  If not
     //! equal, another client has modified the backup.
-    QString etag() const { return loadFromJson<QString>("etag"_ls); }
+    QString etag() const { return loadFromJson<QString>("etag"_L1); }
 
     //! The backup version.
-    QString version() const { return loadFromJson<QString>("version"_ls); }
+    QString version() const { return loadFromJson<QString>("version"_L1); }
+
+    struct Response {
+        //! The algorithm used for storing backups.
+        QString algorithm{};
+
+        //! Algorithm-dependent data. See the documentation for the backup
+        //! algorithms in [Server-side key backups](/client-server-api/#server-side-key-backups) for
+        //! more information on the expected format of the data.
+        QJsonObject authData{};
+
+        //! The number of keys stored in the backup.
+        int count{};
+
+        //! An opaque string representing stored keys in the backup.
+        //! Clients can compare it with the `etag` value they received
+        //! in the request of their last key storage request.  If not
+        //! equal, another client has modified the backup.
+        QString etag{};
+
+        //! The backup version.
+        QString version{};
+    };
+};
+
+template <std::derived_from<GetRoomKeysVersionJob> JobT>
+constexpr inline auto doCollectResponse<JobT> = [](JobT* j) -> GetRoomKeysVersionJob::Response {
+    return { j->algorithm(), j->authData(), j->count(), j->etag(), j->version() };
 };
 
 //! \brief Update information about an existing backup.
@@ -151,7 +208,7 @@ public:
     //!
     //! This function can be used when a URL for DeleteRoomKeysVersionJob
     //! is necessary but the job itself isn't.
-    static QUrl makeRequestUrl(QUrl baseUrl, const QString& version);
+    static QUrl makeRequestUrl(const HomeserverData& hsData, const QString& version);
 };
 
 //! \brief Store a key in the backup.
@@ -177,12 +234,25 @@ public:
 
     //! The new etag value representing stored keys in the backup.
     //! See `GET /room_keys/version/{version}` for more details.
-    QString etag() const { return loadFromJson<QString>("etag"_ls); }
+    QString etag() const { return loadFromJson<QString>("etag"_L1); }
 
     //! The number of keys stored in the backup
-    int count() const { return loadFromJson<int>("count"_ls); }
+    int count() const { return loadFromJson<int>("count"_L1); }
+
+    struct Response {
+        //! The new etag value representing stored keys in the backup.
+        //! See `GET /room_keys/version/{version}` for more details.
+        QString etag{};
+
+        //! The number of keys stored in the backup
+        int count{};
+    };
 };
 
+template <std::derived_from<PutRoomKeyBySessionIdJob> JobT>
+constexpr inline auto doCollectResponse<JobT> =
+    [](JobT* j) -> PutRoomKeyBySessionIdJob::Response { return { j->etag(), j->count() }; };
+
 //! \brief Retrieve a key from the backup.
 //!
 //! Retrieve a key from the backup.
@@ -203,8 +273,8 @@ public:
     //!
     //! This function can be used when a URL for GetRoomKeyBySessionIdJob
     //! is necessary but the job itself isn't.
-    static QUrl makeRequestUrl(QUrl baseUrl, const QString& roomId, const QString& sessionId,
-                               const QString& version);
+    static QUrl makeRequestUrl(const HomeserverData& hsData, const QString& roomId,
+                               const QString& sessionId, const QString& version);
 
     // Result properties
 
@@ -212,6 +282,8 @@ public:
     KeyBackupData data() const { return fromJson<KeyBackupData>(jsonData()); }
 };
 
+inline auto collectResponse(const GetRoomKeyBySessionIdJob* job) { return job->data(); }
+
 //! \brief Delete a key from the backup.
 //!
 //! Delete a key from the backup.
@@ -232,19 +304,32 @@ public:
     //!
     //! This function can be used when a URL for DeleteRoomKeyBySessionIdJob
     //! is necessary but the job itself isn't.
-    static QUrl makeRequestUrl(QUrl baseUrl, const QString& roomId, const QString& sessionId,
-                               const QString& version);
+    static QUrl makeRequestUrl(const HomeserverData& hsData, const QString& roomId,
+                               const QString& sessionId, const QString& version);
 
     // Result properties
 
     //! The new etag value representing stored keys in the backup.
     //! See `GET /room_keys/version/{version}` for more details.
-    QString etag() const { return loadFromJson<QString>("etag"_ls); }
+    QString etag() const { return loadFromJson<QString>("etag"_L1); }
 
     //! The number of keys stored in the backup
-    int count() const { return loadFromJson<int>("count"_ls); }
+    int count() const { return loadFromJson<int>("count"_L1); }
+
+    struct Response {
+        //! The new etag value representing stored keys in the backup.
+        //! See `GET /room_keys/version/{version}` for more details.
+        QString etag{};
+
+        //! The number of keys stored in the backup
+        int count{};
+    };
 };
 
+template <std::derived_from<DeleteRoomKeyBySessionIdJob> JobT>
+constexpr inline auto doCollectResponse<JobT> =
+    [](JobT* j) -> DeleteRoomKeyBySessionIdJob::Response { return { j->etag(), j->count() }; };
+
 //! \brief Store several keys in the backup for a given room.
 //!
 //! Store several keys in the backup for a given room.
@@ -265,12 +350,25 @@ public:
 
     //! The new etag value representing stored keys in the backup.
     //! See `GET /room_keys/version/{version}` for more details.
-    QString etag() const { return loadFromJson<QString>("etag"_ls); }
+    QString etag() const { return loadFromJson<QString>("etag"_L1); }
 
     //! The number of keys stored in the backup
-    int count() const { return loadFromJson<int>("count"_ls); }
+    int count() const { return loadFromJson<int>("count"_L1); }
+
+    struct Response {
+        //! The new etag value representing stored keys in the backup.
+        //! See `GET /room_keys/version/{version}` for more details.
+        QString etag{};
+
+        //! The number of keys stored in the backup
+        int count{};
+    };
 };
 
+template <std::derived_from<PutRoomKeysByRoomIdJob> JobT>
+constexpr inline auto doCollectResponse<JobT> =
+    [](JobT* j) -> PutRoomKeysByRoomIdJob::Response { return { j->etag(), j->count() }; };
+
 //! \brief Retrieve the keys from the backup for a given room.
 //!
 //! Retrieve the keys from the backup for a given room.
@@ -287,7 +385,8 @@ public:
     //!
     //! This function can be used when a URL for GetRoomKeysByRoomIdJob
     //! is necessary but the job itself isn't.
-    static QUrl makeRequestUrl(QUrl baseUrl, const QString& roomId, const QString& version);
+    static QUrl makeRequestUrl(const HomeserverData& hsData, const QString& roomId,
+                               const QString& version);
 
     // Result properties
 
@@ -296,6 +395,8 @@ public:
     RoomKeyBackup data() const { return fromJson<RoomKeyBackup>(jsonData()); }
 };
 
+inline auto collectResponse(const GetRoomKeysByRoomIdJob* job) { return job->data(); }
+
 //! \brief Delete the keys from the backup for a given room.
 //!
 //! Delete the keys from the backup for a given room.
@@ -312,18 +413,32 @@ public:
     //!
     //! This function can be used when a URL for DeleteRoomKeysByRoomIdJob
     //! is necessary but the job itself isn't.
-    static QUrl makeRequestUrl(QUrl baseUrl, const QString& roomId, const QString& version);
+    static QUrl makeRequestUrl(const HomeserverData& hsData, const QString& roomId,
+                               const QString& version);
 
     // Result properties
 
     //! The new etag value representing stored keys in the backup.
     //! See `GET /room_keys/version/{version}` for more details.
-    QString etag() const { return loadFromJson<QString>("etag"_ls); }
+    QString etag() const { return loadFromJson<QString>("etag"_L1); }
 
     //! The number of keys stored in the backup
-    int count() const { return loadFromJson<int>("count"_ls); }
+    int count() const { return loadFromJson<int>("count"_L1); }
+
+    struct Response {
+        //! The new etag value representing stored keys in the backup.
+        //! See `GET /room_keys/version/{version}` for more details.
+        QString etag{};
+
+        //! The number of keys stored in the backup
+        int count{};
+    };
 };
 
+template <std::derived_from<DeleteRoomKeysByRoomIdJob> JobT>
+constexpr inline auto doCollectResponse<JobT> =
+    [](JobT* j) -> DeleteRoomKeysByRoomIdJob::Response { return { j->etag(), j->count() }; };
+
 //! \brief Store several keys in the backup.
 //!
 //! Store several keys in the backup.
@@ -334,18 +449,31 @@ public:
     //!
     //! \param rooms
     //!   A map of room IDs to room key backup data.
-    explicit PutRoomKeysJob(const QString& version, const QHash<QString, RoomKeyBackup>& rooms);
+    explicit PutRoomKeysJob(const QString& version, const QHash<RoomId, RoomKeyBackup>& rooms);
 
     // Result properties
 
     //! The new etag value representing stored keys in the backup.
     //! See `GET /room_keys/version/{version}` for more details.
-    QString etag() const { return loadFromJson<QString>("etag"_ls); }
+    QString etag() const { return loadFromJson<QString>("etag"_L1); }
 
     //! The number of keys stored in the backup
-    int count() const { return loadFromJson<int>("count"_ls); }
+    int count() const { return loadFromJson<int>("count"_L1); }
+
+    struct Response {
+        //! The new etag value representing stored keys in the backup.
+        //! See `GET /room_keys/version/{version}` for more details.
+        QString etag{};
+
+        //! The number of keys stored in the backup
+        int count{};
+    };
 };
 
+template <std::derived_from<PutRoomKeysJob> JobT>
+constexpr inline auto doCollectResponse<JobT> =
+    [](JobT* j) -> PutRoomKeysJob::Response { return { j->etag(), j->count() }; };
+
 //! \brief Retrieve the keys from the backup.
 //!
 //! Retrieve the keys from the backup.
@@ -359,17 +487,19 @@ public:
     //!
     //! This function can be used when a URL for GetRoomKeysJob
     //! is necessary but the job itself isn't.
-    static QUrl makeRequestUrl(QUrl baseUrl, const QString& version);
+    static QUrl makeRequestUrl(const HomeserverData& hsData, const QString& version);
 
     // Result properties
 
     //! A map of room IDs to room key backup data.
-    QHash<QString, RoomKeyBackup> rooms() const
+    QHash<RoomId, RoomKeyBackup> rooms() const
     {
-        return loadFromJson<QHash<QString, RoomKeyBackup>>("rooms"_ls);
+        return loadFromJson<QHash<RoomId, RoomKeyBackup>>("rooms"_L1);
     }
 };
 
+inline auto collectResponse(const GetRoomKeysJob* job) { return job->rooms(); }
+
 //! \brief Delete the keys from the backup.
 //!
 //! Delete the keys from the backup.
@@ -383,16 +513,29 @@ public:
     //!
     //! This function can be used when a URL for DeleteRoomKeysJob
     //! is necessary but the job itself isn't.
-    static QUrl makeRequestUrl(QUrl baseUrl, const QString& version);
+    static QUrl makeRequestUrl(const HomeserverData& hsData, const QString& version);
 
     // Result properties
 
     //! The new etag value representing stored keys in the backup.
     //! See `GET /room_keys/version/{version}` for more details.
-    QString etag() const { return loadFromJson<QString>("etag"_ls); }
+    QString etag() const { return loadFromJson<QString>("etag"_L1); }
 
     //! The number of keys stored in the backup
-    int count() const { return loadFromJson<int>("count"_ls); }
+    int count() const { return loadFromJson<int>("count"_L1); }
+
+    struct Response {
+        //! The new etag value representing stored keys in the backup.
+        //! See `GET /room_keys/version/{version}` for more details.
+        QString etag{};
+
+        //! The number of keys stored in the backup
+        int count{};
+    };
 };
 
+template <std::derived_from<DeleteRoomKeysJob> JobT>
+constexpr inline auto doCollectResponse<JobT> =
+    [](JobT* j) -> DeleteRoomKeysJob::Response { return { j->etag(), j->count() }; };
+
 } // namespace Quotient
