@@ -7,25 +7,25 @@ using namespace Quotient;
 auto queryToGetEventByTimestamp(int ts, const QString& dir)
 {
     QUrlQuery _q;
-    addParam<>(_q, QStringLiteral("ts"), ts);
-    addParam<>(_q, QStringLiteral("dir"), dir);
+    addParam(_q, u"ts"_s, ts);
+    addParam(_q, u"dir"_s, dir);
     return _q;
 }
 
-QUrl GetEventByTimestampJob::makeRequestUrl(QUrl baseUrl, const QString& roomId, int ts,
-                                            const QString& dir)
+QUrl GetEventByTimestampJob::makeRequestUrl(const HomeserverData& hsData, const QString& roomId,
+                                            int ts, const QString& dir)
 {
-    return BaseJob::makeRequestUrl(std::move(baseUrl),
+    return BaseJob::makeRequestUrl(hsData,
                                    makePath("/_matrix/client/v1", "/rooms/", roomId,
                                             "/timestamp_to_event"),
                                    queryToGetEventByTimestamp(ts, dir));
 }
 
 GetEventByTimestampJob::GetEventByTimestampJob(const QString& roomId, int ts, const QString& dir)
-    : BaseJob(HttpVerb::Get, QStringLiteral("GetEventByTimestampJob"),
+    : BaseJob(HttpVerb::Get, u"GetEventByTimestampJob"_s,
               makePath("/_matrix/client/v1", "/rooms/", roomId, "/timestamp_to_event"),
               queryToGetEventByTimestamp(ts, dir))
 {
-    addExpectedKey("event_id");
-    addExpectedKey("origin_server_ts");
+    addExpectedKey(u"event_id"_s);
+    addExpectedKey(u"origin_server_ts"_s);
 }
