@@ -8,8 +8,6 @@
 
 #include <olm/olm.h>
 
-#include <cstring>
-
 using namespace Quotient;
 
 OlmErrorCode QOlmSession::lastErrorCode() const {
@@ -42,7 +40,7 @@ QOlmExpected<QOlmSession> QOlmSession::unpickle(QByteArray&& pickled,
         == olm_error()) {
         const auto errorCode = olmSession.lastErrorCode();
         QOLM_FAIL_OR_LOG_X(errorCode == OLM_OUTPUT_BUFFER_TOO_SMALL,
-                           "Failed to unpickle an Olm session"_ls,
+                           "Failed to unpickle an Olm session"_L1,
                            olmSession.lastError());
         return errorCode;
     }
@@ -91,8 +89,7 @@ QOlmExpected<QByteArray> QOlmSession::decrypt(const QOlmMessage& message) const
                                           unsignedSize(ciphertext),
                                           plaintextBuf.data(), plaintextMaxLen);
     if (actualLength == olm_error()) {
-        QOLM_FAIL_OR_LOG(OLM_OUTPUT_BUFFER_TOO_SMALL,
-                         "Failed to decrypt the message"_ls);
+        QOLM_FAIL_OR_LOG(OLM_OUTPUT_BUFFER_TOO_SMALL, "Failed to decrypt the message"_L1);
         return lastErrorCode();
     }
     // actualLength cannot be more than plainTextLength because the resulting
