@@ -6,7 +6,11 @@
 #include "connection.h"
 #include "database.h"
 #include "logging_categories_p.h"
+#include "csapi/cross_signing.h"
+#include "room.h"
 
+#include "e2ee/cryptoutils.h"
+#include "e2ee/sssshandler.h"
 #include "e2ee/qolmaccount.h"
 
 #include "events/event.h"
@@ -44,35 +48,77 @@ CStructPtr<OlmSAS> KeyVerificationSession::makeOlmData()
     return data;
 }
 
-KeyVerificationSession::KeyVerificationSession(
-    QString remoteUserId, const KeyVerificationRequestEvent& event,
-    Connection* connection, bool encrypted)
+KeyVerificationSession::KeyVerificationSession(QString remoteUserId,
+                                               const KeyVerificationRequestEvent& event,
+                                               Connection* connection, bool encrypted)
+    : KeyVerificationSession(std::move(remoteUserId), connection, event.fromDevice(), encrypted,
+                             event.methods(), event.timestamp(), event.transactionId())
+{}
+
+KeyVerificationSession::KeyVerificationSession(const RoomMessageEvent* event, Room* room)
+    : KeyVerificationSession(event->senderId(), room->connection(),
+                             event->contentPart<QString>("from_device"_L1), room->usesEncryption(),
+                             event->contentPart<QStringList>("methods"_L1),
+                             event->originTimestamp(), {}, room, event->id())
+{}
+
+KeyVerificationSession::KeyVerificationSession(QString remoteUserId, Connection* connection,
+                                               QString remoteDeviceId, bool encrypted,
+                                               QStringList methods, QDateTime startTimestamp,
+                                               QString transactionId, Room* room,
+                                               QString requestEventId)
     : QObject(connection)
-    , m_remoteUserId(std::move(remoteUserId))
-    , m_remoteDeviceId(event.fromDevice())
-    , m_transactionId(event.transactionId())
     , m_connection(connection)
+    , m_room(room)
+    , m_remoteUserId(std::move(remoteUserId))
+    , m_remoteDeviceId(std::move(remoteDeviceId))
+    , m_transactionId(std::move(transactionId))
     , m_encrypted(encrypted)
-    , m_remoteSupportedMethods(event.methods())
+    , m_remoteSupportedMethods(std::move(methods))
+    , m_requestEventId(std::move(requestEventId)) // TODO: Consider merging with transactionId
 {
+    if (m_connection->hasConflictingDeviceIdsAndCrossSigningKeys(m_remoteUserId)) {
+        qCWarning(E2EE) << "Remote user has conflicting device ids and cross signing keys; refusing to verify.";
+        return;
+    }
     const auto& currentTime = QDateTime::currentDateTime();
     const auto timeoutTime =
-        std::min(event.timestamp().addSecs(600), currentTime.addSecs(120));
+        std::min(startTimestamp.addSecs(600), currentTime.addSecs(120));
     const milliseconds timeout{ currentTime.msecsTo(timeoutTime) };
-    if (timeout > 5s)
+    if (timeout > 5s) {
         setupTimeout(timeout);
-    // Otherwise don't even bother starting up
+    } else {
+        // Otherwise don't even bother starting up
+        deleteLater();
+    }
 }
 
 KeyVerificationSession::KeyVerificationSession(QString userId, QString deviceId,
                                                Connection* connection)
+    : KeyVerificationSession(std::move(userId), connection, nullptr, std::move(deviceId),
+                             QUuid::createUuid().toString())
+{}
+
+KeyVerificationSession::KeyVerificationSession(Room* room)
+    : KeyVerificationSession(room->members()[room->members()[0].isLocalMember() ? 1 : 0].id(),
+                             room->connection(),
+                             room)
+{}
+
+KeyVerificationSession::KeyVerificationSession(QString remoteUserId, Connection* connection,
+                                               Room* room, QString remoteDeviceId,
+                                               QString transactionId)
     : QObject(connection)
-    , m_remoteUserId(std::move(userId))
-    , m_remoteDeviceId(std::move(deviceId))
-    , m_transactionId(QUuid::createUuid().toString())
     , m_connection(connection)
-    , m_encrypted(false)
+    , m_room(room)
+    , m_remoteUserId(std::move(remoteUserId))
+    , m_remoteDeviceId(std::move(remoteDeviceId))
+    , m_transactionId(std::move(transactionId))
 {
+    if (m_connection->hasConflictingDeviceIdsAndCrossSigningKeys(m_remoteUserId)) {
+        qCWarning(E2EE) << "Remote user has conflicting device ids and cross signing keys; refusing to verify.";
+        return;
+    }
     setupTimeout(600s);
     QMetaObject::invokeMethod(this, &KeyVerificationSession::sendRequest);
 }
@@ -118,7 +164,12 @@ void KeyVerificationSession::handleEvent(const KeyVerificationEvent& baseEvent)
                     cancelVerification(UNKNOWN_METHOD);
                     return false;
                 }
-                m_commitment = event.commitment();
+                m_commitment = event.commitment().toLatin1();
+                if (!QByteArray::fromBase64Encoding(m_commitment,
+                                                    QByteArray::AbortOnBase64DecodingErrors)) {
+                    cancelVerification(INVALID_MESSAGE);
+                    return false;
+                }
                 sendKey();
                 setState(WAITINGFORKEY);
                 return true;
@@ -143,10 +194,10 @@ struct EmojiStoreEntry : EmojiEntry {
     QHash<QString, QString> translatedDescriptions;
 
     explicit EmojiStoreEntry(const QJsonObject& json)
-        : EmojiEntry{ fromJson<QString>(json["emoji"_ls]),
-                      fromJson<QString>(json["description"_ls]) }
+        : EmojiEntry{ fromJson<QString>(json["emoji"_L1]),
+                      fromJson<QString>(json["description"_L1]) }
         , translatedDescriptions{ fromJson<QHash<QString, QString>>(
-              json["translated_descriptions"_ls]) }
+              json["translated_descriptions"_L1]) }
     {}
 };
 
@@ -155,7 +206,7 @@ using EmojiStore = QVector<EmojiStoreEntry>;
 EmojiStore loadEmojiStore()
 {
     Q_INIT_RESOURCE(libquotientemojis);
-    QFile dataFile(":/sas-emoji.json"_ls);
+    QFile dataFile(":/sas-emoji.json"_L1);
     dataFile.open(QFile::ReadOnly);
     auto data = dataFile.readAll();
     Q_CLEANUP_RESOURCE(libquotientemojis);
@@ -182,12 +233,10 @@ void KeyVerificationSession::handleKey(const KeyVerificationKeyEvent& event)
     olm_sas_set_their_key(olmData, eventKey.data(), unsignedSize(eventKey));
 
     if (startSentByUs) {
-        const auto paddedCommitment =
+        const auto unpaddedCommitment =
             QCryptographicHash::hash((event.key() % m_startEvent).toLatin1(),
                                      QCryptographicHash::Sha256)
-                .toBase64();
-        const QLatin1String unpaddedCommitment(paddedCommitment.constData(),
-                                               QString::fromLatin1(paddedCommitment).indexOf(u'='));
+                .toBase64(QByteArray::OmitTrailingEquals);
         if (unpaddedCommitment != m_commitment) {
             qCWarning(E2EE) << "Commitment mismatch; aborting verification";
             cancelVerification(MISMATCHED_COMMITMENT);
@@ -202,14 +251,15 @@ void KeyVerificationSession::handleKey(const KeyVerificationKeyEvent& event)
 
     std::array<std::byte, 6> output{};
     const auto infoTemplate =
-        startSentByUs ? "MATRIX_KEY_VERIFICATION_SAS|%1|%2|%3|%4|%5|%6|%7"_ls
-                      : "MATRIX_KEY_VERIFICATION_SAS|%4|%5|%6|%1|%2|%3|%7"_ls;
+        startSentByUs ? "MATRIX_KEY_VERIFICATION_SAS|%1|%2|%3|%4|%5|%6|%7"_L1
+                      : "MATRIX_KEY_VERIFICATION_SAS|%4|%5|%6|%1|%2|%3|%7"_L1;
 
     const auto info = infoTemplate
                           .arg(m_connection->userId(), m_connection->deviceId(),
                                QString::fromLatin1(key.data()), m_remoteUserId, m_remoteDeviceId,
-                               event.key(), m_transactionId)
+                               event.key(), m_room ? m_requestEventId : m_transactionId)
                           .toLatin1();
+
     olm_sas_generate_bytes(olmData, info.data(), unsignedSize(info),
                            output.data(), output.size());
 
@@ -243,12 +293,12 @@ QString KeyVerificationSession::calculateMac(const QString& input,
     const auto inputBytes = input.toLatin1();
     const auto macLength = olm_sas_mac_length(olmData);
     auto macChars = byteArrayForOlm(macLength);
-    const auto macInfo =
-        (verifying ? "MATRIX_KEY_VERIFICATION_MAC%3%4%1%2%5%6"_ls
-                   : "MATRIX_KEY_VERIFICATION_MAC%1%2%3%4%5%6"_ls)
-            .arg(m_connection->userId(), m_connection->deviceId(),
-                 m_remoteUserId, m_remoteDeviceId, m_transactionId, keyId)
-            .toLatin1();
+    const auto macInfo = (verifying ? "MATRIX_KEY_VERIFICATION_MAC%3%4%1%2%5%6"_L1
+                                    : "MATRIX_KEY_VERIFICATION_MAC%1%2%3%4%5%6"_L1)
+                             .arg(m_connection->userId(), m_connection->deviceId(), m_remoteUserId,
+                                  m_remoteDeviceId, m_room ? m_requestEventId : m_transactionId,
+                                  input.contains(u',') ? u"KEY_IDS"_s : keyId)
+                             .toLatin1();
     if (m_commonMacCodes.contains(HmacSha256V2Code))
         olm_sas_calculate_mac_fixed_base64(olmData, inputBytes.data(),
                                            unsignedSize(inputBytes),
@@ -263,15 +313,26 @@ QString KeyVerificationSession::calculateMac(const QString& input,
 
 void KeyVerificationSession::sendMac()
 {
-    QString keyId{ "ed25519:"_ls % m_connection->deviceId() };
+    QString keyId{ "ed25519:"_L1 % m_connection->deviceId() };
 
-    auto keys = calculateMac(keyId, false);
+    const auto &masterKey = m_connection->isUserVerified(m_connection->userId()) ? m_connection->masterKeyForUser(m_connection->userId()) : QString();
+
+    QStringList keyList{keyId};
+    if (!masterKey.isEmpty()) {
+        keyList += "ed25519:"_L1 + masterKey;
+    }
+    keyList.sort();
+
+    auto keys = calculateMac(keyList.join(u','), false);
 
     QJsonObject mac;
     auto key = m_connection->olmAccount()->deviceKeys().keys.value(keyId);
     mac[keyId] = calculateMac(key, false, keyId);
+    if (!masterKey.isEmpty()) {
+        mac["ed25519:"_L1 + masterKey] = calculateMac(masterKey, false, "ed25519:"_L1 + masterKey);
+    }
 
-    m_connection->sendToDevice(m_remoteUserId, m_remoteDeviceId,
+    sendEvent(m_remoteUserId, m_remoteDeviceId,
                                KeyVerificationMacEvent(m_transactionId, keys,
                                                        mac),
                                m_encrypted);
@@ -284,7 +345,7 @@ void KeyVerificationSession::sendMac()
 
 void KeyVerificationSession::sendDone()
 {
-    m_connection->sendToDevice(m_remoteUserId, m_remoteDeviceId,
+    sendEvent(m_remoteUserId, m_remoteDeviceId,
                                KeyVerificationDoneEvent(m_transactionId),
                                m_encrypted);
 }
@@ -294,7 +355,7 @@ void KeyVerificationSession::sendKey()
     const auto pubkeyLength = olm_sas_pubkey_length(olmData);
     auto keyBytes = byteArrayForOlm(pubkeyLength);
     olm_sas_get_pubkey(olmData, keyBytes.data(), pubkeyLength);
-    m_connection->sendToDevice(m_remoteUserId, m_remoteDeviceId,
+    sendEvent(m_remoteUserId, m_remoteDeviceId,
                                KeyVerificationKeyEvent(m_transactionId,
                                                        QString::fromLatin1(keyBytes)),
                                m_encrypted);
@@ -303,10 +364,8 @@ void KeyVerificationSession::sendKey()
 
 void KeyVerificationSession::cancelVerification(Error error)
 {
-    m_connection->sendToDevice(m_remoteUserId, m_remoteDeviceId,
-                               KeyVerificationCancelEvent(m_transactionId,
-                                                          errorToString(error)),
-                               m_encrypted);
+    sendEvent(m_remoteUserId, m_remoteDeviceId, KeyVerificationCancelEvent(m_transactionId,
+                                                          errorToString(error)), m_encrypted);
     setState(CANCELED);
     setError(error);
     emit finished();
@@ -322,7 +381,7 @@ void KeyVerificationSession::sendReady()
         return;
     }
 
-    m_connection->sendToDevice(
+    sendEvent(
         m_remoteUserId, m_remoteDeviceId,
         KeyVerificationReadyEvent(m_transactionId, m_connection->deviceId(),
                                   methods),
@@ -338,9 +397,17 @@ void KeyVerificationSession::sendStartSas()
 {
     startSentByUs = true;
     KeyVerificationStartEvent event(m_transactionId, m_connection->deviceId());
+    auto fixedJson = event.contentJson();
+    if (m_room) {
+        fixedJson.remove("transaction_id"_L1);
+        fixedJson["m.relates_to"_L1] = QJsonObject {
+            {"event_id"_L1, m_requestEventId},
+            {"rel_type"_L1, "m.reference"_L1},
+        };
+    }
     m_startEvent = QString::fromUtf8(
-        QJsonDocument(event.contentJson()).toJson(QJsonDocument::Compact));
-    m_connection->sendToDevice(m_remoteUserId, m_remoteDeviceId, event,
+        QJsonDocument(fixedJson).toJson(QJsonDocument::Compact));
+    sendEvent(m_remoteUserId, m_remoteDeviceId, event,
                                m_encrypted);
     setState(WAITINGFORACCEPT);
 }
@@ -351,6 +418,11 @@ void KeyVerificationSession::handleReady(const KeyVerificationReadyEvent& event)
     m_remoteSupportedMethods = event.methods();
     auto methods = commonSupportedMethods(m_remoteSupportedMethods);
 
+    // This happens for outgoing user verification
+    if (m_remoteDeviceId.isEmpty()) {
+        m_remoteDeviceId = event.fromDevice();
+    }
+
     if (methods.isEmpty())
         cancelVerification(UNKNOWN_METHOD);
     else if (methods.size() == 1)
@@ -384,7 +456,7 @@ void KeyVerificationSession::handleStart(const KeyVerificationStartEvent& event)
                                                      QCryptographicHash::Sha256)
                                 .toBase64(QByteArray::OmitTrailingEquals));
 
-    m_connection->sendToDevice(m_remoteUserId, m_remoteDeviceId,
+    sendEvent(m_remoteUserId, m_remoteDeviceId,
                                KeyVerificationAcceptEvent(m_transactionId,
                                                           commitment),
                                m_encrypted);
@@ -395,8 +467,8 @@ void KeyVerificationSession::handleMac(const KeyVerificationMacEvent& event)
 {
     QStringList keys = event.mac().keys();
     keys.sort();
-    const auto& key = keys.join(","_ls);
-    const QString edKeyId = "ed25519:"_ls % m_remoteDeviceId;
+    const auto& key = keys.join(","_L1);
+    const QString edKeyId = "ed25519:"_L1 % m_remoteDeviceId;
 
     if (calculateMac(m_connection->edKeyForUserDevice(m_remoteUserId,
                                                       m_remoteDeviceId),
@@ -406,12 +478,21 @@ void KeyVerificationSession::handleMac(const KeyVerificationMacEvent& event)
         return;
     }
 
+    auto masterKey = m_connection->masterKeyForUser(m_remoteUserId);
+    if (event.mac().contains("ed25519:"_L1 % masterKey)
+        && calculateMac(masterKey, true, "ed25519:"_L1 % masterKey)
+               != event.mac().value("ed25519:"_L1 % masterKey)) {
+        cancelVerification(KEY_MISMATCH);
+        return;
+    }
+
     if (calculateMac(key, true) != event.keys()) {
         cancelVerification(KEY_MISMATCH);
         return;
     }
 
     m_pendingEdKeyId = edKeyId;
+    m_pendingMasterKey = masterKey;
 
     if (m_verified) {
         trustKeys();
@@ -421,6 +502,74 @@ void KeyVerificationSession::handleMac(const KeyVerificationMacEvent& event)
 void KeyVerificationSession::trustKeys()
 {
     m_connection->database()->setSessionVerified(m_pendingEdKeyId);
+    m_connection->database()->setMasterKeyVerified(m_pendingMasterKey);
+    if (m_remoteUserId == m_connection->userId()) {
+        m_connection->reloadDevices();
+    }
+
+    if (!m_pendingMasterKey.isEmpty()) {
+        if (m_remoteUserId == m_connection->userId()) {
+            const auto selfSigningKey = m_connection->database()->loadEncrypted("m.cross_signing.self_signing"_L1);
+            if (!selfSigningKey.isEmpty()) {
+                QHash<QString, QHash<QString, QJsonObject>> signatures;
+                auto json = QJsonObject {
+                    {"keys"_L1, QJsonObject {
+                        {"ed25519:"_L1 + m_remoteDeviceId, m_connection->edKeyForUserDevice(m_remoteUserId, m_remoteDeviceId)},
+                        {"curve25519:"_L1 + m_remoteDeviceId, m_connection->curveKeyForUserDevice(m_remoteUserId, m_remoteDeviceId)},
+                    }},
+                    {"algorithms"_L1, QJsonArray {"m.olm.v1.curve25519-aes-sha2"_L1, "m.megolm.v1.aes-sha2"_L1}},
+                    {"device_id"_L1, m_remoteDeviceId},
+                    {"user_id"_L1, m_remoteUserId},
+                };
+                auto signature = sign(selfSigningKey, QJsonDocument(json).toJson(QJsonDocument::Compact));
+                json["signatures"_L1] = QJsonObject {
+                    {m_connection->userId(), QJsonObject {
+                        {"ed25519:"_L1 + m_connection->database()->selfSigningPublicKey(), QString::fromLatin1(signature)},
+                    }},
+                };
+                signatures[m_remoteUserId][m_remoteDeviceId] = json;
+                auto uploadSignatureJob = m_connection->callApi<UploadCrossSigningSignaturesJob>(signatures);
+                connect(uploadSignatureJob, &BaseJob::finished, m_connection, [uploadSignatureJob]() {
+                    if (uploadSignatureJob->error() != BaseJob::Success) {
+                        qCWarning(E2EE) << "Failed to upload self-signing signature" << uploadSignatureJob << uploadSignatureJob->error() << uploadSignatureJob->errorString();
+                    }
+                });
+            } else {
+                // Not parenting to this since the session is going to be destroyed soon
+                auto handler = new SSSSHandler(m_connection);
+                handler->setConnection(m_connection);
+                handler->unlockSSSSFromCrossSigning();
+                connect(handler, &SSSSHandler::finished, handler, &QObject::deleteLater);
+            }
+        } else {
+            const auto userSigningKey = m_connection->database()->loadEncrypted("m.cross_signing.user_signing"_L1);
+            if (!userSigningKey.isEmpty()) {
+                QHash<QString, QHash<QString, QJsonObject>> signatures;
+                auto json = QJsonObject {
+                    {"keys"_L1, QJsonObject {
+                        {"ed25519:"_L1 + m_pendingMasterKey, m_pendingMasterKey},
+                    }},
+                    {"usage"_L1, QJsonArray {"master"_L1}},
+                    {"user_id"_L1, m_remoteUserId},
+                };
+                auto signature = sign(userSigningKey, QJsonDocument(json).toJson(QJsonDocument::Compact));
+                json["signatures"_L1] = QJsonObject {
+                    {m_connection->userId(), QJsonObject {
+                        {"ed25519:"_L1 + m_connection->database()->userSigningPublicKey(), QString::fromLatin1(signature)},
+                    }},
+                };
+                signatures[m_remoteUserId][m_pendingMasterKey] = json;
+                auto uploadSignatureJob = m_connection->callApi<UploadCrossSigningSignaturesJob>(signatures);
+                connect(uploadSignatureJob, &BaseJob::finished, m_connection, [uploadSignatureJob, userId = m_remoteUserId](){
+                    if (uploadSignatureJob->error() != BaseJob::Success) {
+                        qCWarning(E2EE) << "Failed to upload user-signing signature for" << userId << uploadSignatureJob << uploadSignatureJob->error() << uploadSignatureJob->errorString();
+                    }
+                });
+            }
+        }
+        emit m_connection->userVerified(m_remoteUserId);
+    }
+
     emit m_connection->sessionVerified(m_remoteUserId, m_remoteDeviceId);
     macReceived = true;
 
@@ -439,7 +588,7 @@ QVector<EmojiEntry> KeyVerificationSession::sasEmojis() const
 
 void KeyVerificationSession::sendRequest()
 {
-    m_connection->sendToDevice(
+    sendEvent(
         m_remoteUserId, m_remoteDeviceId,
         KeyVerificationRequestEvent(m_transactionId, m_connection->deviceId(),
                                     supportedMethods,
@@ -475,59 +624,59 @@ QString KeyVerificationSession::errorToString(Error error)
 {
     switch(error) {
         case NONE:
-            return "none"_ls;
+            return "none"_L1;
         case TIMEOUT:
-            return "m.timeout"_ls;
+            return "m.timeout"_L1;
         case USER:
-            return "m.user"_ls;
+            return "m.user"_L1;
         case UNEXPECTED_MESSAGE:
-            return "m.unexpected_message"_ls;
+            return "m.unexpected_message"_L1;
         case UNKNOWN_TRANSACTION:
-            return "m.unknown_transaction"_ls;
+            return "m.unknown_transaction"_L1;
         case UNKNOWN_METHOD:
-            return "m.unknown_method"_ls;
+            return "m.unknown_method"_L1;
         case KEY_MISMATCH:
-            return "m.key_mismatch"_ls;
+            return "m.key_mismatch"_L1;
         case USER_MISMATCH:
-            return "m.user_mismatch"_ls;
+            return "m.user_mismatch"_L1;
         case INVALID_MESSAGE:
-            return "m.invalid_message"_ls;
+            return "m.invalid_message"_L1;
         case SESSION_ACCEPTED:
-            return "m.accepted"_ls;
+            return "m.accepted"_L1;
         case MISMATCHED_COMMITMENT:
-            return "m.mismatched_commitment"_ls;
+            return "m.mismatched_commitment"_L1;
         case MISMATCHED_SAS:
-            return "m.mismatched_sas"_ls;
+            return "m.mismatched_sas"_L1;
         default:
-            return "m.user"_ls;
+            return "m.user"_L1;
     }
 }
 
 KeyVerificationSession::Error KeyVerificationSession::stringToError(const QString& error)
 {
-    if (error == "m.timeout"_ls)
+    if (error == "m.timeout"_L1)
         return REMOTE_TIMEOUT;
-    if (error == "m.user"_ls)
+    if (error == "m.user"_L1)
         return REMOTE_USER;
-    if (error == "m.unexpected_message"_ls)
+    if (error == "m.unexpected_message"_L1)
         return REMOTE_UNEXPECTED_MESSAGE;
-    if (error == "m.unknown_message"_ls)
+    if (error == "m.unknown_message"_L1)
         return REMOTE_UNEXPECTED_MESSAGE;
-    if (error == "m.unknown_transaction"_ls)
+    if (error == "m.unknown_transaction"_L1)
         return REMOTE_UNKNOWN_TRANSACTION;
-    if (error == "m.unknown_method"_ls)
+    if (error == "m.unknown_method"_L1)
         return REMOTE_UNKNOWN_METHOD;
-    if (error == "m.key_mismatch"_ls)
+    if (error == "m.key_mismatch"_L1)
         return REMOTE_KEY_MISMATCH;
-    if (error == "m.user_mismatch"_ls)
+    if (error == "m.user_mismatch"_L1)
         return REMOTE_USER_MISMATCH;
-    if (error == "m.invalid_message"_ls)
+    if (error == "m.invalid_message"_L1)
         return REMOTE_INVALID_MESSAGE;
-    if (error == "m.accepted"_ls)
+    if (error == "m.accepted"_L1)
         return REMOTE_SESSION_ACCEPTED;
-    if (error == "m.mismatched_commitment"_ls)
+    if (error == "m.mismatched_commitment"_L1)
         return REMOTE_MISMATCHED_COMMITMENT;
-    if (error == "m.mismatched_sas"_ls)
+    if (error == "m.mismatched_sas"_L1)
         return REMOTE_MISMATCHED_SAS;
     return NONE;
 }
@@ -541,3 +690,35 @@ QString KeyVerificationSession::transactionId() const
 {
     return m_transactionId;
 }
+
+void KeyVerificationSession::sendEvent(const QString &userId, const QString &deviceId, const KeyVerificationEvent &event, bool encrypted)
+{
+    if (m_room) {
+        auto json = event.contentJson();
+        json.remove("transaction_id"_L1);
+        if (event.metaType().matrixId == KeyVerificationRequestEvent::TypeId) {
+            json["msgtype"_L1] = event.matrixType();
+            json["body"_L1] = m_connection->userId() + " sent a verification request"_L1;
+            json["to"_L1] = m_remoteUserId;
+            m_room->postJson("m.room.message"_L1, json);
+        } else {
+            json["m.relates_to"_L1] = QJsonObject {
+                {"event_id"_L1, m_requestEventId},
+                {"rel_type"_L1, "m.reference"_L1}
+            };
+            m_room->postJson(event.matrixType(), json);
+        }
+    } else {
+        m_connection->sendToDevice(userId, deviceId, event, encrypted);
+    }
+}
+
+bool KeyVerificationSession::userVerification() const
+{
+    return m_room;
+}
+
+void KeyVerificationSession::setRequestEventId(const QString& eventId)
+{
+    m_requestEventId = eventId;
+}
