@@ -5,9 +5,7 @@
 
 #include <QtCore/QBuffer>
 
-#ifdef Quotient_E2EE_ENABLED
 #include "events/filesourceinfo.h"
-#endif
 
 using namespace Quotient;
 
@@ -26,14 +24,15 @@ MxcReply::MxcReply(QNetworkReply* reply,
     connect(d->m_reply, &QNetworkReply::finished, this, [this, fileMetadata] {
         setError(d->m_reply->error(), d->m_reply->errorString());
 
-#ifdef Quotient_E2EE_ENABLED
-        if (fileMetadata.isValid()) {
+        if (d->m_reply->error() != NoError) {
+            const QJsonDocument doc = QJsonDocument::fromJson(d->m_reply->readAll());
+            setErrorString(QStringLiteral("%1 (%2)").arg(doc["error"_L1].toString(), d->m_reply->url().toString()));
+        } else if (fileMetadata.isValid()) {
             auto buffer = new QBuffer(this);
             buffer->setData(decryptFile(d->m_reply->readAll(), fileMetadata));
             buffer->open(ReadOnly);
             d->m_device = buffer;
         }
-#endif
         setOpenMode(ReadOnly);
         emit finished();
     });
@@ -55,10 +54,10 @@ MxcReply::MxcReply()
         }, Qt::QueuedConnection);
 }
 
-qint64 MxcReply::readData(char *data, qint64 maxSize)
+qint64 MxcReply::readData(char *data, qint64 maxlen)
 {
     if(d != nullptr && d->m_device != nullptr) {
-        return d->m_device->read(data, maxSize);
+        return d->m_device->read(data, maxlen);
     }
     return -1;
 }
