@@ -12,12 +12,14 @@
 #include <QtCore/QStringBuilder>
 #include <QtCore/QtEndian>
 
-static const auto RegExpOptions =
+namespace Quotient { // To avoid having to prepend Quotient:: to all the free functions below
+
+inline constexpr auto RegExpOptions =
     QRegularExpression::CaseInsensitiveOption
     | QRegularExpression::UseUnicodePropertiesOption;
 
 // Converts all that looks like a URL into HTML links
-void Quotient::linkifyUrls(QString& htmlEscapedText)
+void linkifyUrls(QString& htmlEscapedText)
 {
     // Note: outer parentheses are a part of C++ raw string delimiters, not of
     // the regex (see http://en.cppreference.com/w/cpp/language/string_literal).
@@ -29,31 +31,26 @@ void Quotient::linkifyUrls(QString& htmlEscapedText)
     // <, >, ' or ", and ends before whitespaces, <, >, ', ", ], !, ), :,
     // comma or dot
     static const QRegularExpression FullUrlRegExp(
-        QStringLiteral(
-            R"(\b((www\.(?!\.)(?!(\w|\.|-)+@)|(https?|ftp):(//)?\w|(magnet|matrix):)(&(?![lg]t;)|[^&\s<>'"])+(&(?![lg]t;)|[^&!,.\s<>'"\]):])))"),
+        uR"(\b((www\.(?!\.)(?!(\w|\.|-)+@)|(https?|ftp):(//)?\w|(magnet|matrix):)(&(?![lg]t;)|[^&\s<>'"])+(&(?![lg]t;)|[^&!,.\s<>'"\]):])))"_s,
         RegExpOptions);
     static const QRegularExpression EmailAddressRegExp(
-        QStringLiteral(R"((^|[][[:space:](){}`'";<>])(mailto:)?((\w|[!#$%&'*+=^_‘{|}~.-])+@(\w|\.|-)+\.\w+\b))"),
+        uR"((^|[][[:space:](){}`'";<>])(mailto:)?((\w|[!#$%&'*+=^_‘{|}~.-])+@(\w|\.|-)+\.\w+\b))"_s,
         RegExpOptions);
     // An interim liberal implementation of
     // https://matrix.org/docs/spec/appendices.html#identifier-grammar
     static const QRegularExpression MxIdRegExp(
-        QStringLiteral(
-            R"((^|[][[:space:](){}`'";])([!#@][-a-z0-9_=#/.]{1,252}:\w(?:\w|\.|-)*\.\w+(?::\d{1,5})?))"),
+        uR"((^|[][[:space:](){}`'";])([!#@][-a-z0-9_=#/.]{1,252}:\w(?:\w|\.|-)*\.\w+(?::\d{1,5})?))"_s,
         RegExpOptions);
-    Q_ASSERT(FullUrlRegExp.isValid() && EmailAddressRegExp.isValid()
-             && MxIdRegExp.isValid());
+    Q_ASSERT(FullUrlRegExp.isValid() && EmailAddressRegExp.isValid() && MxIdRegExp.isValid());
 
     // NOTE: htmlEscapedText is already HTML-escaped! No literal <,>,&,"
 
-    htmlEscapedText.replace(EmailAddressRegExp,
-                            R"(\1<a href='mailto:\3'>\2\3</a>)"_ls);
-    htmlEscapedText.replace(FullUrlRegExp, R"(<a href='\1'>\1</a>)"_ls);
-    htmlEscapedText.replace(MxIdRegExp,
-                            R"(\1<a href='https://matrix.to/#/\2'>\2</a>)"_ls);
+    htmlEscapedText.replace(EmailAddressRegExp, uR"(\1<a href='mailto:\3'>\2\3</a>)"_s);
+    htmlEscapedText.replace(FullUrlRegExp, uR"(<a href='\1'>\1</a>)"_s);
+    htmlEscapedText.replace(MxIdRegExp, uR"(\1<a href='https://matrix.to/#/\2'>\2</a>)"_s);
 }
 
-QString Quotient::sanitized(const QString& plainText)
+QString sanitized(const QString& plainText)
 {
     auto text = plainText;
     text.remove(QChar(0x202e)); // RLO
@@ -62,26 +59,25 @@ QString Quotient::sanitized(const QString& plainText)
     return text;
 }
 
-QString Quotient::prettyPrint(const QString& plainText)
+QString prettyPrint(const QString& plainText)
 {
     auto pt = plainText.toHtmlEscaped();
     linkifyUrls(pt);
-    pt.replace(u'\n', QStringLiteral("<br/>"));
-    return QStringLiteral("<span style='white-space:pre-wrap'>") + pt
-           + QStringLiteral("</span>");
+    pt.replace(u'\n', "<br/>"_L1);
+    return "<span style='white-space:pre-wrap'>"_L1 % pt % "</span>"_L1;
 }
 
-QString Quotient::cacheLocation(const QString& dirName)
+QString cacheLocation(QStringView dirName)
 {
     const QString cachePath =
         QStandardPaths::writableLocation(QStandardPaths::CacheLocation) % u'/'
         % dirName % u'/';
     if (const QDir dir(cachePath); !dir.exists())
-        dir.mkpath("."_ls);
+        dir.mkpath("."_L1);
     return cachePath;
 }
 
-qreal Quotient::stringToHueF(const QString& s)
+qreal stringToHueF(const QString& s)
 {
     Q_ASSERT(!s.isEmpty());
     const auto hash =
@@ -95,15 +91,14 @@ qreal Quotient::stringToHueF(const QString& s)
     return hueF;
 }
 
-static const auto ServerPartRegEx = QStringLiteral(
+inline constexpr auto ServerPartRegEx =
     "(\\[[^][:space:]]+]|[-[:alnum:].]+)" // IPv6 address or hostname/IPv4 address
-    "(?::(\\d{1,5}))?" // Optional port
-);
+    "(?::(\\d{1,5}))?"_L1; // Optional port
 
-QString Quotient::serverPart(const QString& mxId)
+QString serverPart(const QString& mxId)
 {
-    static const QString re("^[@!#$+].*?:("_ls // Localpart and colon
-                            % ServerPartRegEx % ")$"_ls);
+    static const QString re("^[@!#$+].*?:("_L1 // Localpart and colon
+                            % ServerPartRegEx % ")$"_L1);
     static const QRegularExpression parser(
         re,
         QRegularExpression::UseUnicodePropertiesOption); // Because Asian digits
@@ -111,27 +106,26 @@ QString Quotient::serverPart(const QString& mxId)
     return parser.match(mxId).captured(1);
 }
 
-QString Quotient::versionString()
+QString versionString() { return QStringLiteral(Quotient_VERSION_STRING); }
+
+int majorVersion() { return Quotient_VERSION_MAJOR; }
+
+int minorVersion() { return Quotient_VERSION_MINOR; }
+
+int patchVersion() { return Quotient_VERSION_PATCH; }
+
+bool isGuestUserId(const UserId& uId)
 {
-    return QStringLiteral(Quotient_VERSION_STRING);
+    static const QRegularExpression guestMxIdRe{ u"^@\\d+:"_s };
+    return guestMxIdRe.match(uId).hasMatch();
 }
 
-int Quotient::majorVersion()
+bool HomeserverData::checkMatrixSpecVersion(QStringView targetVersion) const
 {
-    return Quotient_VERSION_MAJOR;
+    // TODO: Replace this naïve implementation with something smarter that can check things like
+    //   1.12 > 1.11 and maybe even 1.10 > 1.9
+    return std::ranges::any_of(supportedSpecVersions, [targetVersion](const QString& v) {
+        return v.startsWith(targetVersion);
+    });
 }
-
-int Quotient::minorVersion()
-{
-    return Quotient_VERSION_MINOR;
-}
-
-int Quotient::patchVersion()
-{
-    return Quotient_VERSION_PATCH;
-}
-
-bool Quotient::encryptionSupported()
-{
-    return E2EE_Enabled;
 }
