@@ -3,6 +3,8 @@
 
 #include <Quotient/connection.h>
 #include <Quotient/room.h>
+#include <Quotient/settings.h>
+#include <Quotient/thread.h>
 #include <Quotient/user.h>
 #include <Quotient/uriresolver.h>
 #include <Quotient/networkaccessmanager.h>
@@ -110,6 +112,7 @@ private slots:
     TEST_DECL(addAndRemoveTag)
     TEST_DECL(markDirectChat)
     TEST_DECL(visitResources)
+    TEST_DECL(thread)
     // Add more tests above here
 
 public:
@@ -127,8 +130,8 @@ private:
     template <EventClass<RoomEvent> EventT>
     [[nodiscard]] bool validatePendingEvent(const QString& txnId);
     [[nodiscard]] bool checkDirectChat() const;
-    void finishTest(const TestToken& token, bool condition, const char* file,
-                    int line);
+    void finishTest(const TestToken& token, bool condition,
+                    std::source_location loc = std::source_location::current());
 
 private:
     Room* targetRoom;
@@ -140,11 +143,24 @@ private:
 // Returning true (rather than a void) allows to reuse the convention with
 // connectUntil() to break the QMetaObject::Connection upon finishing the test
 // item.
-#define FINISH_TEST(Condition) \
-    return (finishTest(thisTest, (Condition), __FILE__, __LINE__), true)
+#define FINISH_TEST(Condition) return (finishTest(thisTest, (Condition)), true)
+
+#define FINISH_TEST_IF(Condition) \
+    do {                          \
+        if (Condition)            \
+            FINISH_TEST(true);    \
+    } while (false)
 
 #define FAIL_TEST() FINISH_TEST(false)
 
+#define FAIL_TEST_IF(Condition, ...)                           \
+    do {                                                       \
+        if (Condition) {                                       \
+            __VA_OPT__(clog << QUO_CSTR(__VA_ARGS__) << endl;) \
+            FAIL_TEST();                                       \
+        }                                                      \
+    } while (false)
+
 void TestSuite::doTest(const QByteArray& testName)
 {
     clog << "Starting: " << testName.constData() << endl;
@@ -162,8 +178,7 @@ bool TestSuite::validatePendingEvent(const QString& txnId)
            && (*it)->matrixType() == EventT::TypeId;
 }
 
-void TestSuite::finishTest(const TestToken& token, bool condition,
-                           const char* file, int line)
+void TestSuite::finishTest(const TestToken& token, bool condition, std::source_location loc)
 {
     const auto& item = testName(token);
     if (condition) {
@@ -172,10 +187,11 @@ void TestSuite::finishTest(const TestToken& token, bool condition,
             targetRoom->postMessage(origin % ": "_L1 % QString::fromUtf8(item) % " successful"_L1,
                                     MessageEventType::Notice);
     } else {
-        clog << item << " FAILED at " << file << ":" << line << endl;
+        clog << item << " FAILED at " << loc.file_name() << ":" << loc.line() << endl;
         if (targetRoom)
             targetRoom->postPlainText(origin % ": "_L1 % QString::fromUtf8(item) % " FAILED at "_L1
-                                      % QString::fromUtf8(file) % ", line "_L1 % QString::number(line));
+                                      % QString::fromUtf8(loc.file_name()) % ", line "_L1
+                                      % QString::number(loc.line()));
     }
 
     emit finishedItem(item, condition);
@@ -383,44 +399,43 @@ TEST_IMPL(sendMessage)
 
 TEST_IMPL(sendReaction)
 {
-    clog << "Reacting to the newest message in the room" << endl;
-    Q_ASSERT(targetRoom->timelineSize() > 0);
-    const auto targetEvtId = targetRoom->messageEvents().back()->id();
+    return targetRoom->post<RoomMessageEvent>(u"Reaction target"_s)
+        .whenMerged()
+        .then([this, thisTest](const RoomEvent& targetEvt) {
+            const auto targetEvtId = targetEvt.id();
+            clog << "Reacting to the message just sent to the room: " << targetEvtId.toStdString()
+                 << endl;
 
-    // TODO: a separate test unit for reactionevent.h
-    if (loadEvent<ReactionEvent>(RoomEvent::basicJson(
-            ReactionEvent::TypeId,
-            { { RelatesToKey, toJson(EventRelation::replace(targetEvtId)) } }))) {
-        clog << "ReactionEvent can be created with an invalid relation type"
-             << endl;
-        FAIL_TEST();
-    }
-
-    const auto key = u"+1"_s;
-    const auto txnId = targetRoom->postReaction(targetEvtId, key);
-    if (!validatePendingEvent<ReactionEvent>(txnId)) {
-        clog << "Invalid pending event right after submitting" << endl;
-        FAIL_TEST();
-    }
-
-    connectUntil(targetRoom, &Room::updatedEvent, this,
-        [this, thisTest, txnId, key, targetEvtId](const QString& actualTargetEvtId) {
-            if (actualTargetEvtId != targetEvtId)
-                return false;
-            const auto reactions = targetRoom->relatedEvents(
-                targetEvtId, EventRelation::AnnotationType);
-            // It's a test room, assuming no interference there should
-            // be exactly one reaction
-            if (reactions.size() != 1)
+            // TODO: a separate test unit for reactionevent.h
+            if (loadEvent<ReactionEvent>(RoomEvent::basicJson(
+                    ReactionEvent::TypeId,
+                    { { RelatesToKey, toJson(EventRelation::replace(targetEvtId)) } }))) {
+                clog << "ReactionEvent can be created with an invalid relation type" << endl;
                 FAIL_TEST();
+            }
 
-            const auto* evt =
-                eventCast<const ReactionEvent>(reactions.back());
-            FINISH_TEST(is<ReactionEvent>(*evt) && !evt->id().isEmpty()
-                        && evt->key() == key && evt->transactionId() == txnId);
-            // TODO: Test removing the reaction
-        });
-    return false;
+            const auto key = u"+"_s;
+            const auto txnId = targetRoom->postReaction(targetEvtId, key);
+            FAIL_TEST_IF(!validatePendingEvent<ReactionEvent>(txnId),
+                         "Invalid pending event right after submitting");
+
+            connectUntil(targetRoom, &Room::updatedEvent, this,
+                         [this, thisTest, txnId, key, targetEvtId](const QString& actualTargetEvtId) {
+                             if (actualTargetEvtId != targetEvtId)
+                                 return false;
+                             const auto reactions =
+                                 targetRoom->relatedEvents(targetEvtId,
+                                                           EventRelation::AnnotationType);
+                             FAIL_TEST_IF(reactions.size() != 1);
+
+                             const auto* evt = eventCast<const ReactionEvent>(reactions.back());
+                             FINISH_TEST(is<ReactionEvent>(*evt) && !evt->id().isEmpty()
+                                         && evt->key() == key && evt->transactionId() == txnId);
+                             // TODO: Test removing the reaction
+                         });
+            return false;
+        })
+        .isRunning();
 }
 
 TEST_IMPL(sendFile)
@@ -470,12 +485,13 @@ TEST_IMPL(sendFile)
     return false;
 }
 
-void getResource(const QUrl& url, QScopedPointer<QNetworkReply, QScopedPointerDeleteLater>& r,
-                 QEventLoop& el)
+using NetworkReplyPtr = QObjectHolder<QNetworkReply>;
+
+void getResource(const QUrl& url, NetworkReplyPtr& r, QEventLoop& el)
 {
     r.reset(NetworkAccessManager::instance()->get(QNetworkRequest(url)));
     QObject::connect(
-        r.data(), &QNetworkReply::finished, &el,
+        r.get(), &QNetworkReply::finished, &el,
         [url, &r, &el] {
             if (r->error() != QNetworkReply::NoError)
                 getResource(url, r, el);
@@ -490,7 +506,7 @@ bool testDownload(const QUrl& url)
     // The actual test is separate from the download invocation to help debugging
     const auto results = QtConcurrent::blockingMapped(QVector<int>{ 1, 2, 3 }, [url](int) {
         thread_local QEventLoop el;
-        thread_local QScopedPointer<QNetworkReply, QScopedPointerDeleteLater> reply{};
+        thread_local NetworkReplyPtr reply{};
         getResource(url, reply, el);
         el.exec();
         return reply->error();
@@ -572,6 +588,18 @@ TEST_IMPL(setTopic)
     return false;
 }
 
+// TODO: maybe move it to Room?..
+QFuture<void> ensureEvent(Room* room, const QString& evtId, QPromise<void>&& p = QPromise<void>{})
+{
+    auto future = p.future();
+    if (room->findInTimeline(evtId) == room->historyEdge()) {
+        clog << "Loading a page of history, " << room->timelineSize() << " events so far\n";
+        room->getPreviousContent().then(std::bind_front(ensureEvent, room, evtId, std::move(p)));
+    } else
+        p.finish();
+    return future;
+}
+
 TEST_IMPL(redactEvent)
 {
     using TargetEventType = RoomMemberEvent;
@@ -586,20 +614,25 @@ TEST_IMPL(redactEvent)
     Q_ASSERT(memberEventToRedact); // ...or the room state is totally screwed
     const auto& evtId = memberEventToRedact->id();
 
-    clog << "Redacting the latest member event" << endl;
-    targetRoom->redactEvent(evtId, origin);
-    connectUntil(targetRoom, &Room::addedMessages, this, [this, thisTest, evtId] {
-        auto it = targetRoom->findInTimeline(evtId);
-        if (it == targetRoom->historyEdge())
-            return false; // Waiting for the next sync
-
-        FINISH_TEST((*it)->switchOnType([this](const TargetEventType& e) {
-            return e.redactionReason() == origin
-                   && e.membership() == Membership::Join;
-            // The second condition above tests MSC2176 - if it's violated (pre
-            // 0.8 beta), membership() ends up being Membership::Undefined
-        }));
+    // Make sure the event is loaded in the timeline before proceeding with the test, to make sure
+    // the replacement tracked below actually occurs
+    ensureEvent(targetRoom, evtId).then([this, thisTest, evtId] {
+        clog << "Redacting the latest member event" << endl;
+        targetRoom->redactEvent(evtId, origin);
+        connectUntil(targetRoom, &Room::replacedEvent, this,
+                     [this, thisTest, evtId](const RoomEvent* evt) {
+                         // Concurrent replacement/redaction shouldn't happen as of now; but if/when
+                         // event editing is added to the test suite, this may become a thing
+                         if (evt->id() != evtId)
+                             return false;
+                         FINISH_TEST(evt->switchOnType([this](const TargetEventType& e) {
+                             return e.redactionReason() == origin && e.membership() == Membership::Join;
+                             // The second condition above tests MSC2176 - if it's violated (pre 0.8
+                             // beta), membership() ends up being Membership::Undefined
+                         }));
+                     });
     });
+
     return false;
 }
 
@@ -871,6 +904,32 @@ TEST_IMPL(visitResources)
     FINISH_TEST(true);
 }
 
+TEST_IMPL(thread)
+{
+    auto rootTxnId = targetRoom->postPlainText("Threadroot"_L1);
+    connect(targetRoom, &Room::pendingEventAboutToMerge, this, [this, thisTest, rootTxnId](Quotient::RoomEvent* rootEvt) {
+        if (rootEvt->transactionId() == rootTxnId) {
+            const auto relation = EventRelation::replyInThread(rootEvt->id(), true, rootEvt->id());
+            targetRoom->post<Quotient::RoomMessageEvent>(u"Thread reply 1"_s, Quotient::RoomMessageEvent::MsgType::Text, nullptr, relation)
+                .whenMerged()
+                .then([this, thisTest](const RoomEvent& replyEvt) {
+                    replyEvt.switchOnType(
+                        [&](const RoomMessageEvent& rmReplyEvt) {
+                            const auto thread = targetRoom->threads()[rmReplyEvt.threadRootEventId()];
+                            FINISH_TEST(thread.threadRootId == rmReplyEvt.threadRootEventId() &&
+                                        thread.latestEventId == rmReplyEvt.id() &&
+                                        thread.size == 2
+                            );
+                        },
+                        [this, thisTest](const RoomEvent&) { FAIL_TEST(); }
+                    );
+                });
+        }
+    });
+
+    return false;
+}
+
 bool checkPrettyPrint(
     std::initializer_list<std::pair<const char*, const char*>> tests)
 {
